library(cvTools)

cv.score<- function(tr.data, test.data, c.n){
	if(class(tr.data) != "dist.fun") stop("'tr.data' is of the wrong type.")

	alp.hat <- sum(tr.data$distance>c.n/sqrt(tr.data$n))/tr.data$gridsize
	if(alp.hat >0){
		F.hat <- (tr.data$F.n-(1-alp.hat)*tr.data$F.b)/alp.hat ## Computes the naive estimator of F_s
		F.is=pava(F.hat,tr.data$Freq,decreasing=FALSE) ## Computes the Isotonic Estimator of F_s
		F.is[which(F.is<=0)]=0
		F.is[which(F.is>=1)]=1
		# Fhat.train = alp.hat * F.is + (1-alp.hat) * tr.data$F.b
	} else {
		F.is = tr.data$F.b
	}

	test.data <- sort(test.data) ## Sorts the data set
	test.data.1 <- unique(test.data) ## Finds the unique data points
	test.Fn <- ecdf(test.data) ## Computes the empirical DF of the data
	test.Fn.1 <- test.Fn(test.data.1)
	test.Fb <- pnorm(test.data.1) ######### added pnorm here
	test.Freq <- diff(c(0,test.Fn.1))

	F.is.interpolate <- approx(x = tr.data$F.b, y = F.is, xout = test.Fb, yleft = 0, yright = 1)
	loss <- sum( {alp.hat* F.is.interpolate$y + (1- alp.hat) *test.Fb- test.Fn.1}^2* test.Freq)
	return(loss)
}

cv.mixmodel <- function(data, folds = 10, reps = 1, cn.s = NULL, cn.length = NULL, gridsize = 200){
	if(!is.vector(data)) stop(" 'data' has to be a vector.")
	warning("Make sure that data is transformed such that F_b is N(0,1)")
	n<- length(data)
	
	# cvFolds(length(data), K =folds, R= reps, type= "random")

	if (is.null(cn.s)){
		if( is.null(cn.length)) stop("Both 'cn.s' and 'cn.length' can not be null")
		cn.s <- seq(.001 * log(log(n)), .2* log(log(n)), length.out =cn.length)
	} else if(length(cn.s) != cn.length){
		stop("Length of 'cn.s' is different  from cn.length")
	}
	score <-  rep(0, length(cn.s))
	for( rr in 1:reps){
		cv.ind <- rep(1:folds, times = ceiling(n/folds) )
		cv.ind <- cv.ind[sample.int(length(cv.ind))]
		cv.ind <- cv.ind[1:n]
		for( kk in 1:folds){
			t.1 <- Sys.time()
			t.data <- dist.calc(data[cv.ind != kk], gridsize = gridsize)
			for (cc in 1: length(cn.s)){
				score[cc] <- score[cc] +  cv.score(t.data, test.data = data[cv.ind == kk], c.n = cn.s[cc] ) 
			}
			t.2<- Sys.time()
			if(kk==1) {
				print("Expected time for completion")
				print( folds*reps*(t.2-t.1))
			}
		}
	}
	cn.cv <-  cn.s[which.min(score)]
	tot.out <- dist.calc(data, gridsize = gridsize)
	alp.hat.cv <- sum(tot.out$distance>cn.cv/sqrt(tot.out$n))/tot.out$gridsize
	if(alp.hat.cv >0){
		F.hat <- (tot.out$F.n-(1-alp.hat.cv)*tot.out$F.b)/alp.hat.cv ## Computes the naive estimator of F_s
		Fs.hat=pava(F.hat,tot.out$Freq,decreasing=FALSE) ## Computes the Isotonic Estimator of F_s
		Fs.hat[which(Fs.hat<=0)]=0 # restrict to interval [0,1]
		Fs.hat[which(Fs.hat>=1)]=1 # restrict to interval [0,1]
	} else {
		Fs.hat = tot.out$F.b # if alp.hat.cv = 0, we have no signal, i.e. alternative and null distributions are the same
	}
	Fs.hat.fun<- NULL
	Fs.hat.fun$y = Fs.hat
	Fs.hat.fun$x = tot.out$F.n.x

	ret <-  list(cn.cv = cn.cv,
				 score = score,
				 cn.s = cn.s,
				 Fs.hat = Fs.hat.fun,
				 alp.hat = alp.hat.cv,
				 folds = folds,
				 rep = rep,
				 data =data,
				 dist.out = tot.out )
	ret$call <- match.call()
	class(ret)  <- "cv.mixmodel"

	return(ret)
}
print.cv.mixmodel <- function(x,...){
	cat("Call:")
	print(x$call)
	print(paste("Cross validated estimate of alp is" , x$alp.hat))
	print(paste(" The cross-validated choice of c_n is", x$cn.cv))
}
plot.cv.mixmodel <- function(x,...){
	plot(x$cn.s, x$score, ylab= "cross validation score", xlab= "c_n" )
}

